@@ -175,15 +175,16 @@ def save_df_to_s3(df,
                   file_name,
                   s3_key,
                   s3_bucket_name,
-                  add_date=False):
+                  add_date=False,
+                  header=False):
     """ Saves the dataframe to a gzipped csv file and uploads it to S3."""
     if add_date:
         current_timestamp = dt.datetime.now().strftime('%Y-%m-%d')
         file_name = f'{current_timestamp}/{file_name}'
     gz_buffer = BytesIO()
     with gzip.GzipFile(mode='w', fileobj=gz_buffer) as gz_file:
-        df.to_csv(TextIOWrapper(gz_file, 'utf8'), index=False, header=False, sep='|')
+        df.to_csv(TextIOWrapper(gz_file, 'utf8'), index=False, header=header, sep='|')
     logger.info(
         f'Uploading {file_name} to {s3_bucket_name + "/" + s3_key}')
-    put_object(bucket_name=s3_bucket_name, key=s3_key + file_name,
+    put_object(bucket_name=s3_bucket_name, key=f"{s3_key}/{file_name}",
                content=gz_buffer.getvalue())
