@@ -58,6 +58,14 @@ class KlaviyoClient(CrmClient):
     _KLAVIYO_REQUEST_PROFILE_DELETION_URL = 'https://a.klaviyo.com/api/data-privacy-deletion-jobs/'
     _KLAVIYO_FLOWS_URL = "https://a.klaviyo.com/api/flows/?page[size]=50"
     _KLAVIYO_BULK_UPSERT_PROFILES_URL = 'https://a.klaviyo.com/api/profile-bulk-import-jobs/'
+    
+    # Bulk profile import optimization constants (ONLY for bulk_import_profiles operations)
+    # NOTE: These limits are specific to profile bulk imports. Other bulk operations
+    # (events, subscriptions, etc.) may have different limits.
+    _BULK_PROFILE_IMPORT_MAX_CHUNK_SIZE = 10000  # Maximum profiles per chunk
+    _BULK_PROFILE_IMPORT_MAX_PAYLOAD_SIZE = 5242880  # 5MB in bytes (Klaviyo API limit)
+    _BULK_PROFILE_IMPORT_SAFETY_FACTOR = 0.9  # Use 90% of max to leave safety margin
+    
     _KLAVIYO_LIST_MEMBERS_COUNT = ('https://a.klaviyo.com/api/lists/id?additional-fields[list]'
                                    '=profile_count')
     _KLAVIYO_SEGMENT_MEMBERS_COUNT = ('https://a.klaviyo.com/api/segments/'
@@ -144,7 +152,8 @@ class KlaviyoClient(CrmClient):
                    end_ts: str,
                    metric_name: str,
                    include: str = 'profile',
-                   obj_to_get_from_inc: List = None):
+                   obj_to_get_from_inc: List = None,
+                   metric_id: str = None):
         """ Gets all Klaviyo events for <metric_name>, during the period (<since_ts>, <end_ts>).
 
             Args:
@@ -164,7 +173,8 @@ class KlaviyoClient(CrmClient):
                 KlaviyoError if the metric with the specified name is not found.
         """
         # Get id of metric given the name
-        metric_id = self.get_metric_id(metric_name=metric_name)
+        if metric_id is None:
+            metric_id = self.get_metric_id(metric_name=metric_name)
 
         # Configure URL and URL arguments to get events for <metric_id> since <since_ts>
         url_base = KlaviyoMetric.get_metric_events_url(metric_id)
@@ -783,46 +793,130 @@ class KlaviyoClient(CrmClient):
 
         return successful
 
-    def _start_bulk_import_profile_job(self, format_method: Callable[[Any], dict],
-                                       profiles: List[Any], list_id: str = None) -> bool:
+    def _split_chunk_by_payload_size(
+        self, chunk, format_method, list_id: str = None
+    ):
         """
-        Chunks bulk jobs and posts to klaviyo
+        Recursively split a chunk into smaller sub-chunks if it exceeds the 5MB payload limit.
+        
+        NOTE: This method is specifically designed for bulk profile import operations.
+        
         Args:
-            format_method: method to format the payload correctly
-            profiles: list of profiles to be sent to klaviyo
-            list_id: id of list to insert profiles into (optional)
-
+            chunk: List of profiles to potentially split
+            format_method: Callable to format profile data
+            list_id: Optional list ID for the bulk job
+            
+        Returns:
+            List of chunks that are all under the payload size limit
+        """
+        if not chunk:
+            return [chunk]
+        
+        try:
+            # Create copies of profiles to avoid modifying originals
+            chunk_copy = [profile.copy() for profile in chunk]
+            payload = self._get_payload_for_bulk_job(
+                job_type='profile-bulk-import-job',
+                format_method=format_method,
+                profiles=chunk_copy,
+                list_id=list_id,
+            )
+            payload_size = len(json.dumps(payload))
+            
+            self.logger.info(
+                f'Chunk with {len(chunk)} profiles has payload size {payload_size / 1024 / 1024:.1f}MB'
+            )
+            
+            # If payload is under safety threshold, return as-is
+            max_safe_payload = (self._BULK_PROFILE_IMPORT_MAX_PAYLOAD_SIZE * 
+                              self._BULK_PROFILE_IMPORT_SAFETY_FACTOR)
+            if payload_size <= max_safe_payload:
+                return [chunk]
+            
+            # Calculate how many splits we need based on payload size ratio
+            payload_ratio = payload_size / max_safe_payload
+            num_splits = max(2, int(payload_ratio) + 1)
+            
+            self.logger.info(
+                f'Payload ratio: {payload_ratio:.1f}x, splitting into {num_splits} chunks'
+            )
+            
+            # Split into calculated number of chunks
+            chunk_size = len(chunk) // num_splits
+            result_chunks = []
+            
+            for i in range(0, len(chunk), chunk_size):
+                sub_chunk = chunk[i : i + chunk_size]
+                result_chunks.append(sub_chunk)
+            
+            # Recursively verify and split each sub-chunk if needed
+            final_chunks = []
+            for sub_chunk in result_chunks:
+                final_chunks.extend(
+                    self._split_chunk_by_payload_size(sub_chunk, format_method, list_id)
+                )
+            
+            self.logger.info(f'Final result: {len(final_chunks)} chunks')
+            return final_chunks
+            
+        except Exception as e:
+            self.logger.warning(f'Failed to check payload size: {e}, using original chunk')
+            return [chunk]
+    
+    def _start_bulk_import_profile_job(
+        self, format_method, profiles, list_id = None
+    ):
+        """
+        Optimized version that starts with maximum chunk size (10,000) and recursively splits if needed.
+        
+        Uses the bulk profile import constants to optimize chunking and handle payload size limits.
+        
+        Args:
+            format_method: Callable to format profile data
+            profiles: List of profiles to import
+            list_id: Optional list ID to add profiles to
+            
         Returns:
             True if all jobs complete successfully
-
         """
         job_type = 'profile-bulk-import-job'
         url = self._KLAVIYO_BULK_UPSERT_PROFILES_URL
-
-        # chunk list
-        chunks = chunk_list(profiles, 1000)
-
+        
+        self.logger.info(
+            f'Starting with maximum chunk size: {self._BULK_PROFILE_IMPORT_MAX_CHUNK_SIZE} profiles'
+        )
+        
+        chunks = chunk_list(profiles, self._BULK_PROFILE_IMPORT_MAX_CHUNK_SIZE)
+        
         jobs = []
-
-        # process chunk in chunks
+        
+        # Process chunks and recursively split if necessary
         for n, chunk in enumerate(chunks, 1):
-            self.logger.info(f'Processing chunk {n}...')
-            payload = self._get_payload_for_bulk_job(
-                job_type=job_type,
-                format_method=format_method,
-                profiles=chunk,
-                list_id=list_id
+            self.logger.info(f'Processing chunk {n} with {len(chunk)} profiles...')
+            
+            # Recursively split chunk if it exceeds payload size
+            final_chunks = self._split_chunk_by_payload_size(
+                chunk, format_method, list_id
             )
-
-            job_id = self._submit_bulk_import_profile_job(
-                url=url,
-                payload=payload
-            )
-            jobs.append(job_id)
-            time.sleep(1)
-
+            
+            for sub_chunk_idx, sub_chunk in enumerate(final_chunks):
+                if len(final_chunks) > 1:
+                    self.logger.info(
+                        f'  Processing sub-chunk {sub_chunk_idx + 1}/{len(final_chunks)} with {len(sub_chunk)} profiles'
+                    )
+                
+                payload = self._get_payload_for_bulk_job(
+                    job_type=job_type,
+                    format_method=format_method,
+                    profiles=sub_chunk,
+                    list_id=list_id,
+                )
+                
+                job_id = self._submit_bulk_import_profile_job(url=url, payload=payload)
+                jobs.append(job_id)
+                time.sleep(1)
+        
         self.logger.info(f'Created {len(jobs)} jobs...')
-
         return True
 
     def _get_payload_for_bulk_event_job(self, job_type: str, format_method: Callable[[Any], dict],
@@ -930,15 +1024,44 @@ class KlaviyoClient(CrmClient):
         self.logger.info(f'Got segment with name {segment_name} has {profile_count} members.')
         return segment_name, profile_count
 
+    def _get_campaign_tags(self, campaign_id: str) -> List[Dict]:
+        """
+        Get tags for a specific campaign using the Get Tags for Campaign endpoint
+        
+        Args:
+            campaign_id: The ID of the campaign to get tags for
+            
+        Returns:
+            A list of tag dictionaries
+        """
+        tags_url = f"{self._KLAVIYO_CAMPAIGNS_URL}/{campaign_id}/tags"
+        
+        try:
+            response = self._request(method='GET', url=tags_url)
+            
+            if response.status_code == 200:
+                data = response.json()
+                tags = data.get('data', [])
+                return tags
+            else:
+                self.logger.warning(f"Error getting tags for campaign {campaign_id}: {response.status_code}")
+                return []
+                
+        except Exception as e:
+            self.logger.exception(f"Exception getting tags for campaign {campaign_id}: {e}")
+            return []
+
     def get_email_campaigns(self,
                             include_messages=True) -> Tuple[List[Dict], List[Dict]]:
         """
         Returns a tuple of all campaigns and campaign messages from a Klaviyo site. If
         include_messages = False then only the campaigns are returned.
+        If include_tags = True (default), campaigns will include tag information.
 
         Rate Limits:
             Burst: 10/s
             Steady: 150/m
+            Tags endpoint: Burst: 3/s, Steady: 60/m
         """
         base_url = self._KLAVIYO_CAMPAIGNS_URL
         filter_url = get_filter_url('equals',
@@ -968,4 +1091,24 @@ class KlaviyoClient(CrmClient):
             email_campaigns_url = next_page
             # Sleep for a bit - Klaviyo allows 350 calls per second before a rate limited error
             time.sleep(0.01)
-        return campaign_list, campaign_messages_list
+
+        campaigns_with_tags = []
+        
+        for i, campaign in enumerate(campaign_list, 1):
+            campaign_id = campaign.get('id')
+            campaign_name = campaign.get('attributes', {}).get('name', 'Unknown')
+            
+            self.logger.info(f"Processing campaign {i}/{len(campaign_list)}: {campaign_name} ({campaign_id})")
+            
+            tags = self._get_campaign_tags(campaign_id)
+            
+            campaign_with_tags = campaign.copy()
+            campaign_with_tags['tags'] = tags
+            
+            campaigns_with_tags.append(campaign_with_tags)
+            
+            # Rate limiting for tags endpoint (3/s, 60/m)
+            time.sleep(0.4)  # Wait 400ms between tag requests to respect rate limits
+        
+        self.logger.info(f"Completed fetching tags for {len(campaigns_with_tags)} campaigns")
+        return campaigns_with_tags, campaign_messages_list
