@@ -7,7 +7,7 @@ logger = lambda_logger.get_logger(__name__)
 def post_to_webhook(service: str, webhook_url: str, json: dict) -> bool:
     try:
         response = requests.post(webhook_url, json=json)
-        if not response.status_code == 200:
+        if response.status_code not in [200, 201, 202]:
             logger.error(
                 f'Tried to post message on {service} webhook but failed, status code: {response.status_code}'
             )
