@@ -1,4 +1,4 @@
-from typing import MutableMapping as OrderedDict  # OrderedDict introduced in python version > 3.7.2
+from typing import OrderedDict
 
 import configparser
 
